<?php

namespace Drupal\read_time_calculator\Commands;

use Drush\Commands\DrushCommands;
use Drupal\read_time_calculator\Service\ReadTimeCalculatorService;
use Drupal\node\Entity\Node;

/**
 * Drush commands for Read Time Calculator.
 */
class ReadTimeCalculatorCommands extends DrushCommands {

    protected $readTimeCalculator;

    public function __construct(ReadTimeCalculatorService $read_time_calculator) {
        $this->readTimeCalculator = $read_time_calculator;
    }

    /**
     * Calculate read times for all nodes.
     *
     * @command read-time:calculate-all
     * @aliases rtc-all
     * @usage read-time:calculate-all
     *   Calculate read times for all published nodes.
     */
    public function calculateAll() {
        $query = \Drupal::entityQuery('node')
        ->condition('status', 1)
        ->accessCheck(FALSE);
        
        $nids = $query->execute();
        $nodes = Node::loadMultiple($nids);
        
        $this->output()->writeln(dt('Calculating read times for @count nodes...', ['@count' => count($nodes)]));
        
        foreach ($nodes as $node) {
        $read_time = $this->readTimeCalculator->calculateReadTime($node);
        $this->output()->writeln(dt('Node @nid (@title): @time minutes', [
            '@nid' => $node->id(),
            '@title' => $node->getTitle(),
            '@time' => $read_time,
        ]));
        }
        
        $this->output()->writeln(dt('Completed calculating read times.'));
    }


    /**
     * Calculate read time with caching.
     */
    public function calculateReadTimeWithCache(EntityInterface $entity) {
        $cache_key = 'read_time_calculator:' . $entity->getEntityTypeId() . ':' . $entity->id() . ':' . $entity->getChangedTime();
        
        $cache = \Drupal::cache()->get($cache_key);
        if ($cache && $cache->valid) {
            return $cache->data;
        }

        $read_time = $this->calculateReadTime($entity);
        
        // Cache for 24 hours
        \Drupal::cache()->set($cache_key, $read_time, time() + 86400, [
            'node:' . $entity->id(),
            'read_time_calculator_settings',
        ]);

        return $read_time;
    }

    /**
     * Count words with multilingual support.
     */
    protected function countWordsMultilingual($text, $language = 'en') {
        if (empty($text)) {
            return 0;
        }

        switch ($language) {
            case 'zh':
            case 'ja':
            return mb_strlen(preg_replace('/\s+/', '', $text));

            case 'ko':
            return preg_match_all('/[\p{Hangul}]+/u', $text);

            default:
            $text = preg_replace('/\s+/', ' ', trim($text));
            return str_word_count($text);
        }
    }

    /**
     * Enhanced text extraction with hooks.
     */
    protected function extractTextFromEntity(EntityInterface $entity) {
        // ... existing extraction code ...
        
        // Allow other modules to alter the extracted text
        \Drupal::moduleHandler()->alter('read_time_calculator_text', $text, $entity);
        
        return trim($text);
    }

    /**
     * Calculate with hooks support.
     */
    public function calculateReadTime(EntityInterface $entity) {
        // ... existing calculation code ...
        
        // Allow other modules to alter the calculated read time
        $context = ['entity' => $entity, 'word_count' => $word_count];
        \Drupal::moduleHandler()->alter('read_time_calculator_read_time', $read_time, $entity, $context);
        
        return $read_time;
    }
}