<?php

namespace Drupal\read_time_calculator\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\NodeType;

/**
 * Configure Read Time Calculator settings.
 */
class ReadTimeCalculatorConfigForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['read_time_calculator.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'read_time_calculator_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('read_time_calculator.settings');

    $form['calculation'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Calculation Settings'),
    ];

    $form['calculation']['words_per_minute'] = [
      '#type' => 'number',
      '#title' => $this->t('Words per minute'),
      '#description' => $this->t('Average reading speed in words per minute. Default is 200.'),
      '#default_value' => $config->get('words_per_minute') ?? 200,
      '#min' => 50,
      '#max' => 500,
      '#required' => TRUE,
    ];

    $form['calculation']['round_up'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Round up reading time'),
      '#description' => $this->t('Always round up to the next minute (e.g., 1.2 minutes becomes 2 minutes).'),
      '#default_value' => $config->get('round_up') ?? TRUE,
    ];

    $form['display'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Display Settings'),
    ];

    $form['display']['display_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Display format'),
      '#options' => [
        'minutes' => $this->t('Minutes only (e.g., "5 min read")'),
        'detailed' => $this->t('Detailed (e.g., "5 minutes, 30 seconds")'),
        'approximate' => $this->t('Approximate (e.g., "About 5 minutes")'),
      ],
      '#default_value' => $config->get('display_format') ?? 'minutes',
    ];

    // Color customization settings
    $form['styling'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Color Customization'),
      '#description' => $this->t('Customize the appearance of the read time display.'),
    ];

    $form['styling']['background_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Background Color'),
      '#description' => $this->t('Choose the background color for the read time display.'),
      '#default_value' => $config->get('background_color') ?? '#353535',
    ];

    $form['styling']['text_color'] = [
      '#type' => 'color',
      '#title' => $this->t('Text Color'),
      '#description' => $this->t('Choose the text color for the read time display.'),
      '#default_value' => $config->get('text_color') ?? '#ffffff',
    ];

    $form['styling']['border_radius'] = [
      '#type' => 'number',
      '#title' => $this->t('Border Radius (px)'),
      '#description' => $this->t('Set the border radius for rounded corners. Default is 3px.'),
      '#default_value' => $config->get('border_radius') ?? 3,
      '#min' => 0,
      '#max' => 50,
      '#field_suffix' => 'px',
    ];

    $form['styling']['padding'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Padding'),
      '#description' => $this->t('Set the padding for the read time display. Use CSS format (e.g., "3px 11px 2px 8px").'),
      '#default_value' => $config->get('padding') ?? '3px 11px 2px 8px',
      '#size' => 20,
    ];

    // Preview section
    $form['styling']['preview'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Preview'),
      '#description' => $this->t('Preview of how the read time will appear with your selected colors.'),
    ];

    $preview_bg = $config->get('background_color') ?? '#353535';
    $preview_text = $config->get('text_color') ?? '#ffffff';
    $preview_radius = $config->get('border_radius') ?? 3;
    $preview_padding = $config->get('padding') ?? '3px 11px 2px 8px';

    $form['styling']['preview']['sample'] = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#value' => '📖 5 min read',
      '#attributes' => [
        'class' => ['read-time-preview'],
        'style' => "background-color: {$preview_bg}; color: {$preview_text}; border-radius: {$preview_radius}px; padding: {$preview_padding}; display: inline-block; font-size: 0.9em; font-style: italic;",
      ],
    ];

    $form['content_types'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Content Types'),
      '#description' => $this->t('Select which content types should display read time estimates.'),
    ];

    $node_types = NodeType::loadMultiple();
    $enabled_types = $config->get('enabled_content_types') ?? [];

    $form['content_types']['enabled_content_types'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enabled Content Types'),
      '#options' => [],
      '#default_value' => [],
    ];

    foreach ($node_types as $type) {
      $form['content_types']['enabled_content_types']['#options'][$type->id()] = $type->label();
      if (isset($enabled_types[$type->id()]) && $enabled_types[$type->id()]) {
        $form['content_types']['enabled_content_types']['#default_value'][] = $type->id();
      }
    }

    // Add JavaScript for live preview
    $form['#attached']['library'][] = 'read_time_calculator/admin_form';

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $enabled_types = $form_state->getValue('enabled_content_types') ?? [];
    
    // Filter out unchecked values (which come as 0)
    $enabled_types = array_filter($enabled_types, function($value) {
      return !empty($value);
    });

    $this->config('read_time_calculator.settings')
      ->set('words_per_minute', $form_state->getValue('words_per_minute'))
      ->set('round_up', $form_state->getValue('round_up'))
      ->set('display_format', $form_state->getValue('display_format'))
      ->set('background_color', $form_state->getValue('background_color'))
      ->set('text_color', $form_state->getValue('text_color'))
      ->set('border_radius', $form_state->getValue('border_radius'))
      ->set('padding', $form_state->getValue('padding'))
      ->set('enabled_content_types', $enabled_types)
      ->save();

    parent::submitForm($form, $form_state);

    // Clear cache so changes take effect immediately.
    drupal_flush_all_caches();
    
    $this->messenger()->addMessage($this->t('Read Time Calculator settings have been saved.'));
  }

}