<?php

namespace Drupal\read_time_calculator\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\node\NodeInterface;

/**
 * Service for calculating read time estimates.
 */
class ReadTimeCalculatorService {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a ReadTimeCalculatorService object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * Calculate read time for an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to calculate read time for.
   *
   * @return int
   *   Read time in minutes.
   */
  public function calculateReadTime(EntityInterface $entity) {
    if (!$entity instanceof NodeInterface) {
      return 0;
    }

    $config = $this->configFactory->get('read_time_calculator.settings');
    $words_per_minute = $config->get('words_per_minute') ?? 200;
    $round_up = $config->get('round_up') ?? TRUE;

    $text = $this->extractTextFromEntity($entity);
    $word_count = $this->countWords($text);

    if ($word_count === 0) {
      return 0;
    }

    $minutes = $word_count / $words_per_minute;

    if ($round_up) {
      return (int) ceil($minutes);
    }

    return max(1, (int) round($minutes));
  }

  /**
   * Extract text content from an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity to extract text from.
   *
   * @return string
   *   The extracted text.
   */
  protected function extractTextFromEntity(EntityInterface $entity) {
    $text = '';

    // Get all field definitions.
    $field_definitions = $entity->getFieldDefinitions();

    foreach ($field_definitions as $field_name => $field_definition) {
      // Skip system fields.
      if (strpos($field_name, 'field_') !== 0 && !in_array($field_name, ['title', 'body'])) {
        continue;
      }

      $field_type = $field_definition->getType();

      // Handle different field types.
      if (in_array($field_type, ['text', 'text_long', 'text_with_summary', 'string', 'string_long'])) {
        $field_values = $entity->get($field_name)->getValue();
        
        foreach ($field_values as $value) {
          if (isset($value['value'])) {
            $text .= ' ' . strip_tags($value['value']);
          }
          if (isset($value['summary'])) {
            $text .= ' ' . strip_tags($value['summary']);
          }
        }
      }
    }

    return trim($text);
  }

  /**
   * Count words in text.
   *
   * @param string $text
   *   The text to count words in.
   *
   * @return int
   *   The word count.
   */
  protected function countWords($text) {
    if (empty($text)) {
      return 0;
    }

    // Remove extra whitespace and count words.
    $text = preg_replace('/\s+/', ' ', trim($text));
    
    if (empty($text)) {
      return 0;
    }

    return str_word_count($text);
  }

  /**
   * Format read time for display.
   *
   * @param int $minutes
   *   Read time in minutes.
   *
   * @return string
   *   Formatted read time string.
   */
  public function formatReadTime($minutes) {
    $config = $this->configFactory->get('read_time_calculator.settings');
    $format = $config->get('display_format') ?? 'minutes';

    switch ($format) {
      case 'detailed':
        if ($minutes < 1) {
          return t('Less than 1 minute');
        }
        return t('@count minute@plural', [
          '@count' => $minutes,
          '@plural' => $minutes > 1 ? 's' : '',
        ]);

      case 'approximate':
        return t('About @count min', ['@count' => $minutes]);

      case 'minutes':
      default:
        return t('@count min read', ['@count' => $minutes]);
    }
  }

}